@@ -3,71 +3,70 @@ import functools as ft
 import json
 import logging
 from types import SimpleNamespace
-from typing import Any, Callable, Dict, Generator, Iterable, Optional, Tuple, Type, Union
+from typing import Any, Awaitable, Callable, Dict, Generator, Iterable, Optional, Tuple, Type, Union, cast
+
+from pjrpc import AbstractRequest, AbstractResponse, BatchRequest, BatchResponse, Request, Response, common
+from pjrpc.common import exceptions, generators, v20
+from pjrpc.common.typedefs import JsonRpcRequestId, MethodType
 
-from pjrpc import common, Request, Response, BatchRequest, BatchResponse
-from pjrpc.common import generators, exceptions, v20
 from .tracer import Tracer
 
 logger = logging.getLogger(__package__)
 
 
-class Batch:
+class BaseBatch(abc.ABC):
     """
-    Batch wrapper. Implements some methods to wrap multiple JSON-RPC requests into a single batch request.
+    Base batch wrapper. Implements some methods to wrap multiple JSON-RPC requests into a single batch request.
 
     :param client: JSON-RPC client instance
     """
 
-    class Proxy:
+    class BaseProxy(abc.ABC):
         """
         Proxy object. Provides syntactic sugar to make method calls using dot notation.
 
         :param batch: batch wrapper
         """
 
-        def __init__(self, batch: 'Batch'):
+        def __init__(self, batch: 'BaseBatch'):
             self._batch = batch
 
-        def __getattr__(self, attr: str) -> Callable:
-            def wrapped(*args: Any, **kwargs: Any):
+        def __getattr__(self, attr: str) -> MethodType:
+            def wrapped(*args: Any, **kwargs: Any) -> 'BaseBatch.BaseProxy':
                 self._batch.add(attr, *args, **kwargs)
                 return self
 
             return wrapped
 
-        def __call__(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
+        @abc.abstractmethod
+        def __call__(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Union[Awaitable[Any], Any]:
             """
             Makes an RPC call.
 
             :param _trace_ctx: tracers request context
             """
 
-            return self.call(_trace_ctx)
-
-        def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
+        @abc.abstractmethod
+        def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Union[Awaitable[Any], Any]:
             """
             Makes an RPC call.
 
             :param _trace_ctx: tracers request context
             """
 
-            return self._batch.call(_trace_ctx)
-
     @property
-    def proxy(self) -> 'Proxy':
+    @abc.abstractmethod
+    def proxy(self) -> 'BaseProxy':
         """
         Batch request proxy object.
         """
 
-        return Batch.Proxy(self)
-
-    def __init__(self, client: Union['AbstractClient', 'AbstractAsyncClient']):
+    def __init__(self, client: 'BaseAbstractClient'):
         self._client = client
         self._id_gen = client.id_gen_impl()
         self._requests = client.batch_request_class()
 
-    def __getitem__(self, requests: Iterable[Tuple]) -> Any:
+    def __getitem__(self, requests: Iterable[Tuple]) -> Union[Awaitable[Any], Any]:
         """
         Adds requests to the batch and makes a request.
 
@@ -84,7 +83,7 @@ class Batch:
         ])
         return self.call()
 
-    def __call__(self, method: str, *args: Any, **kwargs: Any) -> 'Batch':
+    def __call__(self, method: str, *args: Any, **kwargs: Any) -> 'BaseBatch':
         """
         Adds the method call to the batch.
 
@@ -96,7 +95,29 @@ class Batch:
 
         return self.add(method, *args, **kwargs)
 
-    def add(self, method: str, *args: Any, **kwargs: Any) -> 'Batch':
+    @abc.abstractmethod
+    def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Union[Awaitable[Any], Any]:
+        """
+        Makes a JSON-RPC request.
+
+        :param _trace_ctx: tracers request context
+        :returns: request results as a tuple
+        """
+
+    @abc.abstractmethod
+    def send(
+        self, request: BatchRequest, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any,
+    ) -> Union[Awaitable[Optional[BatchResponse]], Optional[BatchResponse]]:
+        """
+        Sends a JSON-RPC batch request.
+
+        :param request: request instance
+        :param kwargs: additional client request argument
+        :param _trace_ctx: tracers request context
+        :returns: response instance
+        """
+
+    def add(self, method: str, *args: Any, **kwargs: Any) -> 'BaseBatch':
         """
         Adds the method call to the batch.
 
@@ -111,39 +132,7 @@ class Batch:
         self._requests.append(self._client.request_class(method, args or kwargs, id=next(self._id_gen)))
         return self
 
-    def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
-        """
-        Makes a JSON-RPC request.
-
-        :param _trace_ctx: tracers request context
-        :returns: request results as a tuple
-        """
-
-        response = self.send(self._requests)
-
-        return response.result if response is not None else None
-
-    def send(
-        self, request: BatchRequest, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any,
-    ) -> BatchResponse:
-        """
-        Sends a JSON-RPC batch request.
-
-        :param request: request instance
-        :param kwargs: additional client request argument
-        :param _trace_ctx: tracers request context
-        :returns: response instance
-        """
-
-        return self._client._send(
-            request,
-            response_class=self._client.batch_response_class,
-            validator=self._relate,
-            _trace_ctx=_trace_ctx,
-            **kwargs,
-        )
-
-    def notify(self, method: str, *args: Any, **kwargs: Any) -> 'Batch':
+    def notify(self, method: str, *args: Any, **kwargs: Any) -> 'BaseBatch':
         """
         Adds a notification request to the batch.
 
@@ -174,37 +163,102 @@ class Batch:
                     response = response_map.pop(request.id, None)
                     if response is None and self._client.strict:
                         raise exceptions.IdentityError(f"response '{request.id}' not found")
-                    else:
+                    elif response is not None:
                         response.related = request
 
             if response_map and self._client.strict:
                 raise exceptions.IdentityError(f"unexpected response found: {response_map.keys()}")
 
 
-class AsyncBatch(Batch):
+class Batch(BaseBatch):
+    """
+    Batch wrapper. Implements some methods to wrap multiple JSON-RPC requests into a single batch request.
+
+    :param client: JSON-RPC client instance
+    """
+
+    class Proxy(BaseBatch.BaseProxy):
+
+        def __init__(self, batch: 'Batch'):
+            super().__init__(batch)
+
+        def __call__(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
+            return self.call(_trace_ctx)
+
+        def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
+            return self._batch.call(_trace_ctx)
+
+    @property
+    def proxy(self) -> 'Proxy':
+        return Batch.Proxy(self)
+
+    def __init__(self, client: 'AbstractClient'):
+        super().__init__(client)
+
+    def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Optional[Any]:
+        response = self.send(self._requests)
+
+        return response.result if response is not None else None
+
+    def send(
+        self, request: BatchRequest, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any,
+    ) -> Optional[BatchResponse]:
+        return cast(
+            Optional[BatchResponse], self._client._send(
+                request,
+                response_class=self._client.batch_response_class,
+                validator=self._relate,
+                _trace_ctx=_trace_ctx,
+                **kwargs,
+            ),
+        )
+
+
+class AsyncBatch(BaseBatch):
     """
     Asynchronous batch wrapper. Used to make asynchronous JSON-RPC batch requests.
     """
 
+    class Proxy(BaseBatch.BaseProxy):
+
+        def __init__(self, batch: 'AsyncBatch'):
+            super().__init__(batch)
+
+        async def __call__(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
+            return await self.call(_trace_ctx)
+
+        async def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Any:
+            return await self._batch.call(_trace_ctx)
+
+    @property
+    def proxy(self) -> 'Proxy':
+        return AsyncBatch.Proxy(self)
+
     def __init__(self, client: 'AbstractAsyncClient'):
         super().__init__(client)
 
     async def call(self, _trace_ctx: SimpleNamespace = SimpleNamespace()) -> Optional[Any]:
-        """
-        Makes a JSON-RPC request.
-
-        :param _trace_ctx: tracers request context
-        :returns: request results as a tuple
-        """
-
         response = await self.send(self._requests, _trace_ctx=_trace_ctx)
 
         return response.result if response is not None else None
 
+    async def send(
+        self, request: BatchRequest, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any,
+    ) -> Optional[BatchResponse]:
+        return await cast(
+            Awaitable[Optional[BatchResponse]], self._client._send(
+                request,
+                response_class=self._client.batch_response_class,
+                validator=self._relate,
+                _trace_ctx=_trace_ctx,
+                **kwargs,
+            ),
+        )
 
-class AbstractClient(abc.ABC):
+
+class BaseAbstractClient(abc.ABC):
     """
-    Abstract JSON-RPC client.
+    Base abstract JSON-RPC client.
 
     :param request_class: request class
     :param response_class: response class
@@ -226,38 +280,12 @@ class AbstractClient(abc.ABC):
         :param client: JSON-RPC client instance
         """
 
-        def __init__(self, client: 'AbstractClient'):
+        def __init__(self, client: 'BaseAbstractClient'):
             self._client = client
 
         def __getattr__(self, attr: str) -> Callable:
             return ft.partial(self._client.call, attr)
 
-    @property
-    def proxy(self) -> 'Proxy':
-        """
-        Clint proxy object.
-        """
-
-        return AbstractClient.Proxy(self)
-
-    @property
-    def batch(self) -> Batch:
-        """
-        Client batch wrapper.
-        """
-
-        return Batch(self)
-
-    @abc.abstractmethod
-    def _request(self, request_text: str, is_notification: bool = False, **kwargs: Any) -> Optional[str]:
-        """
-        Makes a JSON-RPC request.
-
-        :param request_text: request text representation
-        :param is_notification: is the request a notification
-        :returns: response text representation
-        """
-
     def __init__(
         self,
         request_class: Type[Request] = v20.Request,
@@ -265,7 +293,7 @@ class AbstractClient(abc.ABC):
         batch_request_class: Type[BatchRequest] = v20.BatchRequest,
         batch_response_class: Type[BatchResponse] = v20.BatchResponse,
         error_cls: Type[exceptions.JsonRpcError] = exceptions.JsonRpcError,
-        id_gen_impl: Callable[..., Generator[Union[int, str], None, None]] = generators.sequential,
+        id_gen_impl: Callable[..., Generator[JsonRpcRequestId, None, None]] = generators.sequential,
         json_loader: Callable = json.loads,
         json_dumper: Callable = json.dumps,
         json_encoder: Type[common.JSONEncoder] = common.JSONEncoder,
@@ -288,7 +316,13 @@ class AbstractClient(abc.ABC):
         self._request_args = request_args or {}
         self._tracers = tracers
 
-    def __call__(self, method: str, *args: Any, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any) -> Any:
+    def __call__(
+        self,
+        method: str,
+        *args: Any,
+        _trace_ctx: SimpleNamespace = SimpleNamespace(),
+        **kwargs: Any,
+    ) -> Union[Awaitable[Any], Any]:
         """
         Makes JSON-RPC call.
 
@@ -301,7 +335,75 @@ class AbstractClient(abc.ABC):
 
         return self.call(method, *args, **kwargs)
 
-    def notify(self, method: str, *args: Any, _trace_ctx=SimpleNamespace(), **kwargs: Any):
+    @property
+    def proxy(self) -> 'Proxy':
+        """
+        Clint proxy object.
+        """
+
+        return BaseAbstractClient.Proxy(self)
+
+    @abc.abstractmethod
+    def call(self, method: str, *args: Any, **kwargs: Any) -> Union[Awaitable[Any], Any]:
+        pass
+
+    @abc.abstractmethod
+    def _send(
+        self,
+        request: AbstractRequest,
+        response_class: Type[AbstractResponse],
+        validator: Callable[..., None],
+        _trace_ctx: SimpleNamespace = SimpleNamespace(),
+        **kwargs: Any,
+    ) -> Union[Awaitable[Optional[AbstractResponse]], Optional[AbstractResponse]]:
+        pass
+
+    def _relate(self, request: Request, response: Response) -> None:
+        """
+        Checks the the request and the response identifiers match.
+
+        :param request: request
+        :param response: response
+        """
+
+        if self.strict and response.id is not None and response.id != request.id:
+            raise exceptions.IdentityError(
+                f"response id doesn't match the request one: expected {request.id}, got {response.id}",
+            )
+
+        response.related = request
+
+
+class AbstractClient(BaseAbstractClient):
+    """
+    Abstract synchronous JSON-RPC client.
+    """
+
+    @property
+    def batch(self) -> Batch:
+        """
+        Client batch wrapper.
+        """
+
+        return Batch(self)
+
+    @abc.abstractmethod
+    def _request(self, request_text: str, is_notification: bool = False, **kwargs: Any) -> Optional[str]:
+        """
+        Makes a JSON-RPC request.
+
+        :param request_text: request text representation
+        :param is_notification: is the request a notification
+        :returns: response text representation
+        """
+
+    def notify(
+        self,
+        method: str,
+        *args: Any,
+        _trace_ctx: SimpleNamespace = SimpleNamespace(),
+        **kwargs: Any,
+    ) -> Optional[Response]:
         """
         Makes a notification request
 
@@ -322,7 +424,7 @@ class AbstractClient(abc.ABC):
 
     def call(
         self, method: str, *args: Any, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any,
-    ) -> Optional[Response]:
+    ) -> Any:
         """
         Makes JSON-RPC call.
 
@@ -342,6 +444,7 @@ class AbstractClient(abc.ABC):
         )
         response = self.send(request, _trace_ctx=_trace_ctx)
 
+        assert response is not None, "response is not set"
         return response.result
 
     def send(
@@ -356,17 +459,24 @@ class AbstractClient(abc.ABC):
         :returns: response instance
         """
 
-        return self._send(
-            request,
-            response_class=self.response_class,
-            validator=self._relate,
-            _trace_ctx=_trace_ctx,
-            **kwargs,
+        return cast(
+            Response, self._send(
+                request,
+                response_class=self.response_class,
+                validator=self._relate,
+                _trace_ctx=_trace_ctx,
+                **kwargs,
+            ),
         )
 
-    def traced(method: Callable) -> Callable:
+    def traced(method: Callable[..., Any]) -> Callable[..., Any]:
         @ft.wraps(method)
-        def wrapper(self, request, _trace_ctx, **kwargs):
+        def wrapper(
+            self: 'AbstractClient',
+            request: AbstractRequest,
+            _trace_ctx: SimpleNamespace,
+            **kwargs: Any,
+        ) -> Optional[AbstractResponse]:
             for tracer in self._tracers:
                 tracer.on_request_begin(_trace_ctx, request)
 
@@ -387,12 +497,12 @@ class AbstractClient(abc.ABC):
     @traced
     def _send(
         self,
-        request: Request,
-        response_class: Type[Response],
-        validator: Callable,
+        request: AbstractRequest,
+        response_class: Type[AbstractResponse],
+        validator: Callable[..., None],
         _trace_ctx: SimpleNamespace = SimpleNamespace(),
         **kwargs: Any,
-    ) -> Optional[Response]:
+    ) -> Optional[AbstractResponse]:
         kwargs = {**self._request_args, **kwargs}
         request_text = self.json_dumper(request, cls=self.json_encoder)
 
@@ -410,23 +520,8 @@ class AbstractClient(abc.ABC):
 
         return response
 
-    def _relate(self, request: Request, response: Response) -> None:
-        """
-        Checks the the request and the response identifiers match.
 
-        :param request: request
-        :param response: response
-        """
-
-        if self.strict and response.id is not None and response.id != request.id:
-            raise exceptions.IdentityError(
-                f"response id doesn't match the request one: expected {request.id}, got {response.id}",
-            )
-
-        response.related = request
-
-
-class AbstractAsyncClient(AbstractClient):
+class AbstractAsyncClient(BaseAbstractClient):
     """
     Abstract asynchronous JSON-RPC client.
     """
@@ -461,13 +556,20 @@ class AbstractAsyncClient(AbstractClient):
         :returns: response instance
         """
 
-        return await self._send(
-            request, _trace_ctx=_trace_ctx, response_class=self.response_class, validator=self._relate, **kwargs,
+        return cast(
+            Response, await self._send(
+                request, _trace_ctx=_trace_ctx, response_class=self.response_class, validator=self._relate, **kwargs,
+            ),
         )
 
-    def traced(method: Callable) -> Callable:
+    def traced(method: Callable[..., Any]) -> Callable[..., Any]:
         @ft.wraps(method)
-        async def wrapper(self, request: Request, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any):
+        async def wrapper(
+            self: 'AbstractAsyncClient',
+            request: Request,
+            _trace_ctx: SimpleNamespace = SimpleNamespace(),
+            **kwargs: Any,
+        ) -> Response:
             for tracer in self._tracers:
                 tracer.on_request_begin(_trace_ctx, request)
 
@@ -488,12 +590,12 @@ class AbstractAsyncClient(AbstractClient):
     @traced
     async def _send(
         self,
-        request: Request,
-        response_class: Type[Response],
-        validator: Callable,
+        request: AbstractRequest,
+        response_class: Type[AbstractResponse],
+        validator: Callable[..., None],
         _trace_ctx: SimpleNamespace = SimpleNamespace(),
         **kwargs: Any,
-    ) -> Optional[Response]:
+    ) -> Optional[AbstractResponse]:
         kwargs = {**self._request_args, **kwargs}
         request_text = self.json_dumper(request, cls=self.json_encoder)
 
@@ -511,6 +613,31 @@ class AbstractAsyncClient(AbstractClient):
 
         return response
 
+    async def notify(
+        self,
+        method: str,
+        *args: Any,
+        _trace_ctx: SimpleNamespace = SimpleNamespace(),
+        **kwargs: Any,
+    ) -> Optional[Response]:
+        """
+        Makes a notification request
+
+        :param method: method name
+        :param args: method positional arguments
+        :param kwargs: method named arguments
+        :param _trace_ctx: tracers request context
+        """
+
+        assert not (args and kwargs), "positional and keyword arguments are mutually exclusive"
+
+        request = self.request_class(
+            id=None,
+            method=method,
+            params=args or kwargs,
+        )
+        return await self.send(request, _trace_ctx=_trace_ctx)
+
     async def call(
         self, method: str, *args: Any, _trace_ctx: SimpleNamespace = SimpleNamespace(), **kwargs: Any,
     ) -> Any:
@@ -533,4 +660,5 @@ class AbstractAsyncClient(AbstractClient):
         )
         response = await self.send(request, _trace_ctx=_trace_ctx)
 
+        assert response is not None, "response is not set"
         return response.result
