@@ -7,17 +7,19 @@ import asyncio
 import collections
 import functools as ft
 import json
-import sys
 import unittest.mock
-from typing import Any, Callable, Dict, Optional, Union
+from types import ModuleType
+from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Union
 
 import pytest
 
 import pjrpc
 from pjrpc import Response
 from pjrpc.common import UNSET, UnsetType
+from pjrpc.common.typedefs import JsonRpcParams, JsonRpcRequestId
 
-IS_GE_PY38 = sys.version_info >= (3, 8)
+CallType = Dict[Tuple[str, str], unittest.mock.Mock]
+MatchType = Dict[Tuple[str, str], List['Match']]
 
 
 class Match:
@@ -51,18 +53,18 @@ class PjRpcMocker:
     :param passthrough: pass not mocked requests to the original method
     """
 
-    def __init__(self, target, mocker=unittest.mock, passthrough: bool = False):
+    def __init__(self, target: str, mocker: ModuleType = unittest.mock, passthrough: bool = False):
         self._target = target
         self._mocker = mocker
-        self._patcher = None
+        self._patcher: Optional[Any] = None
         self._async_resp = False
         self._passthrough = passthrough
 
-        self._matches: Dict = collections.defaultdict(lambda: collections.defaultdict(list))
-        self._calls: Dict = collections.defaultdict(dict)
+        self._matches: Dict[str, MatchType] = collections.defaultdict(lambda: collections.defaultdict(list))
+        self._calls: Dict[str, CallType] = collections.defaultdict(dict)
 
     @property
-    def calls(self) -> Dict:
+    def calls(self) -> Dict[str, CallType]:
         """
         Dictionary of JSON-PRC method calls.
         """
@@ -75,7 +77,7 @@ class PjRpcMocker:
         method_name: str,
         result: UnsetType = UNSET,
         error: UnsetType = UNSET,
-        id: Optional[Union[int, str]] = None,
+        id: Optional[JsonRpcRequestId] = None,
         version: str = '2.0',
         once: bool = False,
         callback: Optional[Callable] = None,
@@ -102,12 +104,12 @@ class PjRpcMocker:
         method_name: str,
         result: UnsetType = UNSET,
         error: UnsetType = UNSET,
-        id: Optional[Union[int, str]] = None,
+        id: Optional[JsonRpcRequestId] = None,
         version: str = '2.0',
         once: bool = False,
         callback: Optional[Callable] = None,
         idx: int = 0,
-    ):
+    ) -> None:
         """
         Replaces a previously added response patch by a new one.
 
@@ -125,7 +127,12 @@ class PjRpcMocker:
         match = Match(endpoint, version, method_name, once, id=id, result=result, error=error, callback=callback)
         self._matches[endpoint][(version, method_name)][idx] = match
 
-    def remove(self, endpoint: str, method_name: Optional[str] = None, version: str = '2.0'):
+    def remove(
+        self,
+        endpoint: str,
+        method_name: Optional[str] = None,
+        version: str = '2.0',
+    ) -> Union[MatchType, List[Match]]:
         """
         Removes a previously added response patch.
 
@@ -136,6 +143,7 @@ class PjRpcMocker:
         :returns: removed response patch
         """
 
+        result: Union[MatchType, List[Match]]
         if method_name is None:
             result = self._matches.pop(endpoint)
         else:
@@ -157,36 +165,54 @@ class PjRpcMocker:
 
         self._calls.clear()
 
-    def start(self):
+    def start(self) -> Any:
         """
         Activates a patcher.
         """
 
-        self._patcher = self._mocker.patch(self._target, side_effect=self._on_request, autospec=True)
+        patcher = self._mocker.patch(self._target)
+        with patcher:
+            if asyncio.iscoroutinefunction(patcher.temp_original):
+                self._async_resp = True
 
-        result = self._patcher.start()
-        if asyncio.iscoroutinefunction(self._patcher.temp_original):
-            self._async_resp = True
+        side_effect: Callable
+        if self._async_resp:
+            async def side_effect(*args: Any, **kwargs: Any) -> str:
+                return await self._on_request(*args, **kwargs)
+        else:
+            def side_effect(*args: Any, **kwargs: Any) -> str:
+                return self._on_request(*args, **kwargs)
 
-        return result
+        self._patcher = self._mocker.patch(self._target, side_effect=side_effect, autospec=True)
+
+        assert self._patcher is not None
+        return self._patcher.start()
 
     def stop(self) -> None:
         """
         Stop an active patcher.
         """
 
+        assert self._patcher is not None, 'mocker is not started'
+
         self.reset()
         self._patcher.stop()
 
     def _cleanup_matches(self, endpoint: str, version: str = '2.0', method_name: Optional[str] = None) -> None:
-        matches = self._matches[endpoint].get((version, method_name))
+        matches: Optional[Union[MatchType, List[Match]]]
+        if method_name is not None:
+            matches = self._matches[endpoint].get((version, method_name))
+        else:
+            matches = self._matches[endpoint]
 
-        if not matches:
+        if not matches and method_name:
             self._matches[endpoint].pop((version, method_name), None)
         if not self._matches[endpoint]:
             self._matches.pop(endpoint)
 
-    def _on_request(self, origin_self: Any, request_text: str, is_notification: bool = False, **kwargs: Any) -> str:
+    def _on_request(self, origin_self: Any, request_text: str, is_notification: bool = False, **kwargs: Any) -> Any:
+        assert self._patcher is not None, 'mocker is not started'
+
         endpoint = origin_self._endpoint
         matches = self._matches.get(endpoint)
         if matches is None:
@@ -197,6 +223,7 @@ class PjRpcMocker:
 
         json_data = json.loads(request_text)
 
+        response: Union[pjrpc.BatchResponse, pjrpc.Response]
         if isinstance(json_data, (list, tuple)):
             response = pjrpc.BatchResponse()
             for request in pjrpc.BatchRequest.from_json(json_data):
@@ -208,8 +235,8 @@ class PjRpcMocker:
             request = pjrpc.Request.from_json(json_data)
             response = self._match_request(endpoint, request.version, request.method, request.params, request.id)
 
-        if not IS_GE_PY38 and self._async_resp:
-            async def wrapper():
+        if self._async_resp:
+            async def wrapper() -> str:
                 return json.dumps(response.to_json())
             return wrapper()
         else:
@@ -220,8 +247,8 @@ class PjRpcMocker:
         endpoint: str,
         version: str,
         method_name: str,
-        params: Optional[Union[list, dict]],
-        id: Optional[Union[int, str]],
+        params: Optional[JsonRpcParams],
+        id: Optional[JsonRpcRequestId],
     ) -> Response:
         matches = self._matches[endpoint].get((version, method_name))
         if matches is None:
@@ -239,14 +266,18 @@ class PjRpcMocker:
         )
         if isinstance(params, (list, tuple)):
             stub(*params)
-        else:
+        elif isinstance(params, dict):
             stub(**params)
+        else:
+            stub(params)
 
         if match.callback:
             if isinstance(params, (list, tuple)):
                 result = match.callback(*params)
-            else:
+            elif isinstance(params, dict):
                 result = match.callback(**params)
+            else:
+                result = match.callback(params)
 
             return pjrpc.Response(id=id, result=result)
 
@@ -257,11 +288,11 @@ class PjRpcMocker:
                 error=match.response_data['error'],
             )
 
-    def __enter__(self):
+    def __enter__(self) -> 'PjRpcMocker':
         self.start()
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, *args: Any) -> None:
         self.stop()
         self.reset()
 
@@ -272,7 +303,7 @@ PjRpcAiohttpMocker = ft.partial(PjRpcMocker, target='pjrpc.client.backend.aiohtt
 
 
 @pytest.fixture
-def pjrpc_requests_mocker():
+def pjrpc_requests_mocker() -> Generator[PjRpcMocker, None, None]:
     """
     Requests client mocking fixture.
     """
@@ -282,7 +313,7 @@ def pjrpc_requests_mocker():
 
 
 @pytest.fixture
-def pjrpc_aiohttp_mocker():
+def pjrpc_aiohttp_mocker() -> Generator[PjRpcMocker, None, None]:
     """
     Aiohttp client mocking fixture.
     """
