@@ -1,8 +1,8 @@
 import logging
 from types import SimpleNamespace
-from typing import Union
+from typing import Optional
 
-from pjrpc import BatchRequest, Request, Response
+from pjrpc import AbstractRequest, AbstractResponse
 
 client_logger = logging.getLogger(__package__)
 
@@ -12,7 +12,7 @@ class Tracer:
     JSON-RPC client tracer.
     """
 
-    def on_request_begin(self, trace_context: SimpleNamespace, request: Request) -> None:
+    def on_request_begin(self, trace_context: SimpleNamespace, request: AbstractRequest) -> None:
         """
         Handler called before JSON-RPC request begins.
 
@@ -20,7 +20,9 @@ class Tracer:
         :param request: JSON-RPC request
         """
 
-    def on_request_end(self, trace_context: SimpleNamespace, request: Request, response: Response) -> None:
+    def on_request_end(
+        self, trace_context: SimpleNamespace, request: AbstractRequest, response: Optional[AbstractResponse],
+    ) -> None:
         """
         Handler called after JSON-RPC request ends.
 
@@ -30,7 +32,7 @@ class Tracer:
         """
 
     def on_error(
-        self, trace_context: SimpleNamespace, request: Union[Request, BatchRequest], error: BaseException,
+        self, trace_context: SimpleNamespace, request: AbstractRequest, error: BaseException,
     ) -> None:
         """
         Handler called when JSON-RPC request failed.
@@ -50,13 +52,15 @@ class LoggingTracer(Tracer):
         self._logger = logger
         self._level = level
 
-    def on_request_begin(self, trace_context: SimpleNamespace, request: Request) -> None:
+    def on_request_begin(self, trace_context: SimpleNamespace, request: AbstractRequest) -> None:
         self._logger.log(self._level, "sending request: %r", request)
 
-    def on_request_end(self, trace_context: SimpleNamespace, request: Request, response: Response) -> None:
+    def on_request_end(
+        self, trace_context: SimpleNamespace, request: AbstractRequest, response: Optional[AbstractResponse],
+    ) -> None:
         self._logger.log(self._level, "received response: %r", response)
 
     def on_error(
-        self, trace_context: SimpleNamespace, request: Union[Request, BatchRequest], error: BaseException,
+        self, trace_context: SimpleNamespace, request: AbstractRequest, error: BaseException,
     ) -> None:
         self._logger.log(self._level, "request '%s' sending error: %r", request, error)
